from os import path
from urlparse import urlsplit
from urlreduce import utils
from urlreduce.models import Link
from django.utils.encoding import smart_unicode


def replace_insensitive(string, target, replacement):
    """
    Similar to string.replace() but is case insensitive
    Code borrowed from: http://forums.devshed.com/python-programming-11/case-insensitive-string-replace-490921.html
    """
    no_case = string.lower()
    index = no_case.rfind(target.lower())
    if index >= 0:
        return string[:index] + replacement + string[index + len(target):]
    else: # no results so return the original string
        return string

class UrlShortnerMiddleware:
    """
    The URL shortner middleware handles creating, and injecting short urls into
    your webpages. This is done by generating the url, creating the Link response
    header for your HTML document and inject the html meta option into your HTML
    page.
    """

    # -- Holds the Link Object
    link        = None
    shortlink   = None

    def process_request( self, request ):
        
        # -- Ignore Everything in Media
        if request.get_full_path().split( "/" )[1] != "media":
            
            # -- Define host, make sure there is no www to make it shorter
            host = request.get_host().replace( "www.", "" )
            
            # -- Get Link Record from the Database
            try:
                self.link = Link.objects.get( url = request.build_absolute_uri().rstrip('/') )
            except Link.DoesNotExist:
                self.link = Link( url = request.build_absolute_uri().rstrip('/') ).save()
            
            # -- Construct Shortlink
            if request.is_secure():
                self.shortlink = "https://%s/x/%s" % ( host, self.link.short_url )
            else:
                self.shortlink = "http://%s/x/%s" % ( host, self.link.short_url )
            
            # -- Attach to Request
            request.META['shortlink'] = self.shortlink
            
    def process_response( self, request, response ):
        """
        Injection Pattern Based on django-debug-toolbar:
        https://github.com/robhudson/django-debug-toolbar
        """
        
        FIND        = "</head>"
        REPLACE     = '\t<link rel="shortlink" href="%s" />\n</head>'
        HTML_TYPES  = ('text/html', 'application/xhtml+xml')
        
        # -- Check if 200 and HTML Request Type
        if response.status_code == 200:
            if response['Content-Type'].split(';')[0] in HTML_TYPES:
                
                # -- Add Link to Request Headers
                response['Link'] = '<%s>; rel="shortlink"' % self.shortlink
             
                # -- Inject into HTML Head
                response.content = replace_insensitive( smart_unicode(response.content), FIND, REPLACE % self.shortlink )
                
                # -- Refresh Content Length
                if response.get('Content-Length', None):
                    response['Content-Length'] = len( response.content )

        return response


from django.contrib import admin

from urlreduce.models import Link

class LinkAdmin( admin.ModelAdmin ):
    """
    Class Defining the Admin Interface for Shortened URL's
    """

    fieldsets = (
        ( "Site", { 'fields': ( 'url', 'short_url' ) } ),
    )
    
    list_display    = [ 'url', 'short_url', 'key' ]
    
    class Meta:
        ordering = [ 'url' ]

admin.site.register( Link, LinkAdmin )


from django.db import models
from urlreduce import utils

class Link( models.Model ):
    """
    Link Holds a base URI and a shortened URI, this can be used for both local
    and remote sites.
    """
    
    url         = models.URLField( verify_exists = False, unique = True )
    short_url   = models.URLField( verify_exists = False, blank = True, unique = True )

    def key( self ):
        return utils.base62_encode( self.pk )

    def save( self, *args, **kwargs ):
        """
        Save Override Handles Generating the Short URL
        """
        
        # -- Save to Generate PK
        super( Link, self ).save(*args, **kwargs)
        
        # -- Generate Short URL
        if self.short_url == "" or self.short_url == None:
            self.short_url = utils.base62_encode( self.pk )
            super( Link, self ).save(*args, **kwargs)

    def __unicode__( self ):
        return self.short_url


def shortlink( request ):
    """
    Context Processor provides the shortlink URL to the view as SHORTLINK.
    """

    c = {
        'SHORTLINK' : request.META['shortlink'] ,
    }
    
    return c


ALPHABET = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

def base62_encode( num, alphabet = ALPHABET ):
    """
    Encode a number in Base X
    Author - Baishampayan Ghose (http://stackoverflow.com/users/8024)

    `num`: The number to encode
    `alphabet`: The alphabet to use for encoding
    """
    if (num == 0):
        return alphabet[0]
    arr = []
    base = len(alphabet)
    while num:
        rem = num % base
        num = num // base
        arr.append(alphabet[rem])
    arr.reverse()
    return ''.join(arr)

def base62_decode( string, alphabet = ALPHABET ):
    """
    Decode a Base X encoded string into the number

    Author - Baishampayan Ghose (http://stackoverflow.com/users/8024)

    Arguments:
    - `string`: The encoded string
    - `alphabet`: The alphabet to use for encoding
    """
    base = len(alphabet)
    strlen = len(string)
    num = 0

    idx = 0
    for char in string:
        power = (strlen - (idx + 1))
        num += alphabet.index(char) * (base ** power)
        idx += 1

    return num


from django.http import Http404, HttpResponseNotAllowed, HttpResponseRedirect
from django.template import RequestContext

from urlreduce.models import Link


def forward( request, id ):
    """
    View Definition for the Homepage
    """
    if request.method == 'GET':  
        try:
            link = Link.objects.get( short_url = id )
            return HttpResponseRedirect( link.url )
        except Link.DoesNotExist:
            return Http404( "Shortlink does not exist for the requested url: %s" % url )
    else:
        return HttpResponseNotAllowed( ['GET'] )

    


from setuptools import setup, find_packages

setup(
    name                    = 'django-url-reduce',
    version                 = '0.1.3',
    description             = 'A Django pluggable app that automatically generates and injects shortlinks into your webpages.',
    author                  = 'Kieran Lynn',
    author_email            = 'kieran@octothorpstudio.com',
    license                 = open('LICENSE').read(),
    url                     = 'https://github.com/octothorp/django-url-reduce',
    download_url            = 'https://github.com/octothorp/django-url-reduce/downloads',
    packages                = find_packages(),
    include_package_data    = True,
    zip_safe                = True,
    classifiers             = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)